;---------------------------------------------------------------;
;						   07/10/89 BEW ;
;	    ASSEMBLY LANGUAGE EXAMPLE PROGRAM			;
;								;
; Here is a short example program which reads (and displays)	;
;   time code from a PC-LTC/RDR or PC-LTC/RG1 board.		;
;   [Note - Compile using Microsoft Macro Assembler first]	;
; The program terminates on any keypress.			;
; From this example you can get some idea of what is required 	;
;   to properly interface your software to a PC-LTC board.	;
; If desired, you may "borrow" portions of this program for use	;
;   in your own software.					;
; Be sure to look at the "LTC_COMMAND" procedure at the end	;
;   of this file, which is a good example of how to interface	;
;   to PC-LTC boards using POLLING (not interrupts).		;
; There are no examples yet for using interrupts.		;
;---------------------------------------------------------------;
VIDEO	EQU	10H		;VIDEO BIOS INTERRUPT.
KEYBD	EQU	16H		;KEYBOARD BIOS INTERRUPT.
DOS_FN	EQU	21H		;DOS FUNCTION INTERRUPT.
CR	EQU	0DH		;CARRIAGE RETURN CHARACTER.
LF	EQU	0AH		;LINE FEED CHARACTER.

CODE 	SEGMENT
	ASSUME CS:CODE,DS:CODE
	ORG 0100H		;REQUIRED FOR .COM FILES.

X_EXAMPLE PROC FAR
START:	PUSH DS
	PUSH ES
	PUSH AX
	PUSH BX
	PUSH CX
	PUSH SI
	MOV AX,CS
	MOV DS,AX		;SET DS TO EQUAL CS.
	MOV AX,0CC00H		;PUT PC-LTC BOARD ADDRESS HERE.
	MOV ES,AX

	MOV BYTE PTR ES:[3FFH],02  ;SEND RESET COMMAND TO BOARD.

	MOV SI,OFFSET HEADER	;POINT TO HEADER (TITLE) STRING.
	CALL WRSTRING		;DISPLAY THE STRING (ALSO GIVES DELAY).

LOOP1:	CMP BYTE PTR ES:[3FDH],00
	JE LOOP2
	MOV AL,ES:[3FEH]	;DO DUMMY READ OF 3FEH.
	MOV BYTE PTR ES:[3FDH],00
	MOV BYTE PTR ES:[3FFH],00
	MOV CX,1000H
	LOOP $			;DELAY.
	JMP LOOP1		;CLEAR ALL PENDING INTERRUPTS.

LOOP2:	CALL KEYON?
	JNZ EXIT
	CMP BYTE PTR ES:[3FDH],00
	JE LOOP2		   ;WAIT FOR NONZERO VALUE (OR KEYPRESS).
	MOV AX,ES:[3FEH]	   ;GET INTERRUPT CODE.
	MOV BYTE PTR ES:[3FDH],00  ;INIT FOR NEXT TIME.
	MOV BYTE PTR ES:[3FFH],00  ;ACKNOWLEDGE THE INTERRUPT.
	CMP AL,13H		;TC & UB READY?
	JNE LOOP2
DATAOK:	MOV BX,ES:[0]		;BH = SS, BL = FF.
	MOV CX,ES:[2]		;CH = HH, CL = MM.
	MOV AL,CR
	CALL WRCHAR		;RETURN TO START OF LINE.
	MOV AL,CH
	AND AL,3FH
	CALL WR_BCD_2		;PRINT HOURS.
	MOV AL,':'
	CALL WRCHAR
	MOV AL,CL
	AND AL,7FH
	CALL WR_BCD_2		;PRINT MINUTES.
	MOV AL,':'
	CALL WRCHAR
	MOV AL,BH
	AND AL,7FH
	CALL WR_BCD_2		;PRINT SECONDS.
	MOV AL,':'
	CALL WRCHAR
	MOV AL,BL
	AND AL,3FH
	CALL WR_BCD_2		;PRINT FRAMES.
	JMP LOOP1

EXIT:	MOV AH,0
	INT KEYBD		;ELSE READ KEY CHAR AND EXIT.
	POP SI
	POP CX
	POP BX
	POP AX
	POP ES
	POP DS
	INT 20H			;RETURN TO DOS.
X_EXAMPLE ENDP


HEADER  DB CR,LF,LF,LF,LF,'Adrienne Electronics Corporation',CR,LF
	DB 'PC-LTC Time Code Reader Example Program',CR,LF
	DB 'HH:MM:SS:FF$'


KEYON? PROC NEAR
	PUSH AX
	MOV AH,1
	INT KEYBD	;SETS/CLEARS ZF.
	POP AX
	RET
KEYON? ENDP


WR_BCD_2 PROC NEAR	;DISPLAYS THE TWO BCD DIGITS IN AL.
	PUSH AX
	MOV AH,AL
	ROR AL,1
	ROR AL,1
	ROR AL,1
	ROR AL,1
	AND AX,0F0FH
	OR AX,3030H
	CALL WRCHAR
	MOV AL,AH
	CALL WRCHAR
	POP AX
	RET
WR_BCD_2 ENDP


WRCHAR PROC NEAR	;WRITES THE CHAR IN AL TO THE SCREEN.
	PUSH AX
	PUSH BX
	MOV AH,14D
	MOV BL,0FH
	INT VIDEO
	POP BX
	POP AX
	RET
WRCHAR ENDP


WRSTRING PROC NEAR	;WRITES STRING WHICH BEGINS AT DS:SI.
	PUSH AX
	PUSH SI
	CLD
WS_LP:	LODSB
	CMP AL,'$'
	JE WS_XT
	CALL WRCHAR
	JMP WS_LP
WS_XT:	POP SI
	POP AX
	RET
WRSTRING ENDP


PAGE
;NOTE - ALTHOUGH THE FOLLOWING PROCEDURE IS NOT USED IN THE SIMPLE
;       PROGRAM ABOVE, IT IS A MORE GENERAL PURPOSE COMMAND INTERFACE,
;       AND IS USED AS PART OF OUR "PCLTCTST" PROGRAM.
;---------------------------------------------------------------;
; 						    6/16/88 BEW ;
;	      SEND A COMMAND TO THE PC-LTC BOARD		;
;								;
; If AH is zero, sends the command in AL to the PC-LTC board.	;
; Else sends nothing.						;
; Then waits up to 50ms for a response equal to AL.		;
; Returns nonzero if there is an error.				;
; Saves all registers.						;
;---------------------------------------------------------------;
LTC_COMMAND PROC NEAR
	PUSH AX
	PUSH CX

SLC_WR:	OR AH,AH
	JNZ SLC_RD		;DON'T SEND IF AH NONZERO.
	CALL WAIT_FOR_FF	;WAIT FOR PORT 3FFH TO BECOME FFH
	MOV ES:[3FFH],AL	;  THEN SEND THE COMMAND BYTE.

SLC_RD:	MOV CX,50D
SLC_LP:	TEST BYTE PTR ES:[3FDH],0FFH
	JNZ SLC_RSP
;	CALL DELAY_1MS		;NEED A 1MS DELAY HERE.
	LOOP SLC_LP		;50 TIMES MAX => 50MS MAX.
SLC_BAD:OR AL,80H		;SETS ZF=0.
	JMP SHORT SLC_XT

SLC_RSP:MOV AH,ES:[3FEH]		;READ INTERRUPT SOURCE CODE.
	CALL WAIT_FOR_FF		;WAIT FOR PORT 3FFH TO BECOME FFH.
	MOV BYTE PTR ES:[3FDH],00H	;CLEAR FOR NEXT TIME.
	MOV BYTE PTR ES:[3FFH],00H	;CLEAR THE INTERRUPT, ENABLE NEXT ONE.
	CMP AH,AL
	JNE SLC_LP		;IF EQUAL, EXIT WITH ZF=1.

SLC_XT:	POP CX
	POP AX
	RET
LTC_COMMAND ENDP


WAIT_FOR_FF PROC NEAR		;WAITS FOR FFH AT DPRAM PORT 3FFH.
	PUSH CX
	MOV CX,0		;LOOP TIMEOUT COUNTER.
WFF_LP:	CMP BYTE PTR ES:[3FFH],0FFH	;HAS ACKNOWLEDGEMENT ARRIVED?
	LOOPNE WFF_LP		;NO => KEEP LOOPING.
WFF_XT:	POP CX
	RET
WAIT_FOR_FF ENDP



CODE ENDS
END START
